( cd usr/lib ; rm -rf libgstcontrol-0.6.so )
( cd usr/lib ; ln -sf libgstcontrol-0.6.so.0.0.0 libgstcontrol-0.6.so )
( cd usr/lib ; rm -rf libgstcontrol-0.6.so.0 )
( cd usr/lib ; ln -sf libgstcontrol-0.6.so.0.0.0 libgstcontrol-0.6.so.0 )
( cd usr/lib ; rm -rf libgstreamer-0.6.so )
( cd usr/lib ; ln -sf libgstreamer-0.6.so.0.0.0 libgstreamer-0.6.so )
( cd usr/lib ; rm -rf libgstreamer-0.6.so.0 )
( cd usr/lib ; ln -sf libgstreamer-0.6.so.0.0.0 libgstreamer-0.6.so.0 )
